# Copyright (C) 2019 
#
# The source code in this file is released under the GPLv3.
# Go to https://www.gnu.org/licenses/gpl.txt for the full license details.
#
# No, I'm not going to clean this mess up! :D

import upygame as pygame
import urandom as random
import umachine

import res
import sfx

pygame.display.init()
pygame.display.set_palette_16bit([
    0, 6438, 18917, 10825, 47398, 688, 41764, 17475,
    58225, 13598, 60486, 40179, 42596, 46845, 63245, 65535
])

screen_w = 110
screen_h = 88
mixer = pygame.mixer.Sound()

class Drawable:
    positions = ((0,0),)
    images = res.cat_images
    loopy = False
    wait_frames = 60
    image = None
    x = None
    y = None
    max_state = 1

    def __init__(self):
        self.cur_state = 0
        self.set_state(0)
    
    def update(self):
        pass

    def reset(self):
        self.set_state(0)

    def draw(self, surface):
        surface.blit(self.image, self.x, self.y)
    
    def set_state(self, state):
        if state >= 0 and state <= self.max_state:
            self.cur_state = state
        elif self.loopy:
            if state > self.max_state:
                self.cur_state = 0
            else:
                self.cur_state = self.max_state

        self.set_image()
        self.x = self.positions[self.cur_state][0]
        self.y = self.positions[self.cur_state][1]
    
    def set_image(self):
        self.image = self.images[self.cur_state]
    
    def increase_state(self):
        self.set_state(self.cur_state + 1)
    
    def decrease_state(self):
        self.set_state(self.cur_state - 1)


class Cat(Drawable):
    # NOTE: These attributes are shared among ALL instances of cats
    #       but this is ok, because we will only have one cat =^..^=
    positions = ((90,68),(60,68), (40,68), (40,68))
    images = res.cat_images
    max_state = 3
    
    def set_state(self, state):
        # this code is stupid...
        if state == 0:  # step 1
            mixer.play_sfx(sfx.pos_0, len(sfx.pos_0), True)
        elif state == 1: # step 2
            mixer.play_sfx(sfx.pos_1, len(sfx.pos_1), True)
        elif state == 2: # step 3
            mixer.play_sfx(sfx.pos_2, len(sfx.pos_2), True)
        elif state == 3: # ded
            mixer.play_sfx(sfx.dead, len(sfx.dead), True)
        Drawable.set_state(self, state)


class Lion(Drawable):
    positions = ((5,65),(5,65),(10,65))
    cur_state = 0
    images = res.lion_images
    reaction_time = 10 # miliseconds to DEATHTRIGGER
    react_countdown = 30
    max_state = 2
   
    def sleep(self):
        self.set_state(0)
        self.react_countdown = self.reaction_time
        self.wait_frames = 60 + random.getrandbits(4)
    
    def peek(self):
        self.set_state(1) # peek    
        self.wait_frames = 10 + random.getrandbits(4)

    def update(self):
        self.wait_frames -= 1
        if self.wait_frames < 0:
            if self.cur_state == 0:
                self.peek()
            else:
                self.sleep()
    
    def react(self):
        self.react_countdown -= 1
        return self.react_countdown < 0
    
    def levelup(self):
        if self.react_countdown > 2:
            self.react_countdown -= 1

class Sun(Drawable):
    positions = (
        (10, 50), (20, 30), (40, 20), (65, 40), (80, 60), # sun
        (10, 60), (20, 35), (50, 20), (65, 40), (80, 50), # moon
    )
    cur_state = 0
    images = res.sun_images
    loopy = True
    moon_phase = 0
    unlocked_moon_phase = 1
    max_state = 9
    
    def get_sky_color(self):
        if self.cur_state < 3:
            return 9 # skyblue
        return 5 # nightsky
    
    def set_image(self):
        if self.cur_state == 0:
            # daybreak
            self.image = self.images[0]
        elif self.cur_state == 5: 
            # dusk
            self.image = self.images[1 + self.moon_phase]
            self.moon_phase += 1
            if self.moon_phase > self.unlocked_moon_phase:
                self.moon_phase = 0
    
    def add_moon_phase(self):
        if self.unlocked_moon_phase < 5:
            self.unlocked_moon_phase += 1


# init game stuff
screen = pygame.display.set_mode()

# eh, whatever.. may as well put the start screen here..
blinky_count = 0
mixer.play_sfx(sfx.title, len(sfx.title), True)

while True:
    # handle events
    eventtype = pygame.event.poll()
    if eventtype != pygame.NOEVENT and eventtype.type == pygame.KEYUP and eventtype.key == pygame.BUT_A:
        break
    screen.fill(9, pygame.Rect(0,0,110,68)) 
    screen.fill(7, pygame.Rect(0,68,110,20)) 
    #screen.blit(res.cat_images[2], 40, 68)
    screen.blit(res.logo_image, 40, 30)
    umachine.draw_text(1, 21, "KameraKatze!", 1)
    umachine.draw_text(2, 20, "KameraKatze!", 15)
    if blinky_count > 30:
        umachine.draw_text(25, 60, "Press A", 1)
        umachine.draw_text(26, 59, "Press A", 15)
    
    pygame.display.flip()
    
    blinky_count += 1
    if blinky_count > 60:
        blinky_count = 0

# drawables
cat = Cat()
lion = Lion()
sun = Sun()
drawables = (cat, lion)

cur_score = 0
highscore = 0
game_over = 0
bonus_count = 0
multiplier = 1

flash = False
# very messy mainloop
while True:
    if game_over > 5:
        umachine.wait(3000)
        for d in drawables:
            d.reset()
        sun.reset()
        if cur_score > highscore:
            higscore = cur_score
        cur_score = 0
        game_over = False
        bonus_count = 0
    
    # handle events
    eventtype = pygame.event.poll()
    if not game_over and eventtype != pygame.NOEVENT:
        if eventtype.type == pygame.KEYUP:
            if eventtype.key == pygame.K_RIGHT:
                cat.decrease_state()
                multiplier = 1  # reset multiplier
            elif eventtype.key == pygame.K_LEFT and cat.cur_state < 2:
                cat.increase_state()
            elif eventtype.key == pygame.BUT_A and cat.cur_state == 2:
                old_score = cur_score
                cur_score = old_score + 10 * multiplier
                multiplier += 1  # daring kittehs get moar points
                
                flash = True
                mixer.play_sfx(sfx.photo_shoot, len(sfx.photo_shoot), True)

                # increase states 
                if cur_score // 200 - old_score // 200 > 0:
                    sun.increase_state()  # move sun/moon
                
                if cur_score // 400 - old_score // 400 > 0:
                    bonus_count += 1

                if cur_score // 1000 - old_score // 1000 > 0:
                    lion.levelup()
                    mixer.play_sfx(sfx.levelup, len(sfx.levelup), True)
                    sun.add_moon_phase()
    
    # kitteh ded?
    if cat.cur_state == 2 and lion.cur_state == 1 and lion.react():
        cat.set_state(3)
        lion.set_state(2)
        game_over = 1

    # draw sky
    screen.fill(sun.get_sky_color(), pygame.Rect(0,0,110,68)) 
    # draw stars at night
    if sun.cur_state > 2:
        pos_x = 0
        pos_y = 10
        for star_idx in range(bonus_count):
            screen.blit(res.star_images[star_idx % len(res.star_images)], pos_x, pos_y)
            pos_x += 15
            if pos_x > 105:
                pos_y += 15
                pos_x = 0 
                if pos_y > 60:
                    break
    sun.draw(screen)
    
    # draw grass 
    screen.fill(7, pygame.Rect(0,68,110,20)) 
    pos_x = 100
    pos_y = 67
    for flower_idx in range(bonus_count):
        screen.blit(res.flower_images[flower_idx % len(res.flower_images)], pos_x, pos_y)
        pos_x -= 10
        if pos_x < 0:
            pos_x = 100
            pos_y += 5
            if pos_y > 60: 
                break

    
    # draw fg
    umachine.draw_text(2, 2, str(cur_score), 1)
    umachine.draw_text(3, 1, str(cur_score), 15)
    if game_over:
        umachine.draw_text(15, 20, "Game Over!", 1)
        umachine.draw_text(16, 19, "Game Over!", 15)
        cat.x += 2
        game_over += 1
    if game_over < 3:
        for d in drawables:
            d.update()
    for d in drawables:
        d.draw(screen)
    if flash:
        flash = False
        screen.blit(res.flash_images[random.getrandbits(2)], 35, 78)

    pygame.display.flip()